/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package messageproducerclient;

import java.io.IOException;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.Destination;
import javax.jms.JMSException;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueConnectionFactory;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import com.sun.messaging.ConnectionConfiguration;
import javax.jms.QueueSender;

/**
 * Simple class to populate the input queue with test data, either from
 * swift message data file or with simple text test messages, using
 * the same GlassFish queue connection factory which is created when you
 * deploy StatelessSessionConsumerEJB.
 *
 * @author slossr1
 */
public class MessagePopulator {

    //logger for class
    Logger logger = Logger.getLogger(MessagePopulator.class.getName());

    //used only when populating simple test messages
    int numOfMessages = 1000;

    //used for GF population
    InitialContext incx;
    QueueConnectionFactory connectionFactory;
    QueueConnection connection;
    QueueSession session;
    Queue q;
    MessageProducer messageProducer;
    TextMessage message;

    //used for direct MQ population
    static final String inputFile = "10000_swift_messages.txt";
    static final String openMQHost = "localhost";
    static final String openMQPort = "7676";
    QueueSender messageSender;
    QueueSession sess;
    QueueConnection conn;

    public static void main(String[] args) {
        MessagePopulator mp = new MessagePopulator();
        /*  uncomment the line below and comment out the mp.populateFromTestDataFile
            to populate programmatically with simple test messages
         */
        //mp.populateWithTestMessages();
        /*  uncomment the line below and comment out the mp.populateFromTestDataFile(false)
            to populate from the swift messages file using GlassFish connection factory
         */
        //mp.populateFromTestDataFile(true);
        mp.populateFromTestDataFile(false);
        System.exit(0);
    }

    /**
     * Populates INPUT.Q from delimited test data file 10000_swift_messages.txt
     * @param useGFConnFact boolean to indicate if should write to input queue directly
     *  or use GlassFish connection factory
     */
    private void populateFromTestDataFile(boolean useGFConnFact) {
        try {
            logger.log(Level.INFO,
                "about to populate INPUT.Q using "+inputFile);
            InputStream input = this.getClass().getResourceAsStream(inputFile);
            if(input==null) {
                throw new IOException("could not load "+inputFile);
            }
            List<String> messages = getMessagesFromFile(input, "$");
            if(useGFConnFact) {
                populateWithGFConnectionFactory(messages);
            } else {
                populateUsingMQConnection(messages);
            }
            logger.log(Level.INFO,
                    "completed writing "+numOfMessages+" to the input.q");
        } catch (IOException ex) {
            logger.log(Level.SEVERE, null, ex);
        } catch (NamingException ex) {
            logger.log(Level.SEVERE, null, ex);
        } catch (JMSException ex) {
            logger.log(Level.SEVERE, null, ex);
        } finally {
            if(useGFConnFact) {
                closeGFResources();
            } else {
                closeMQResources();
            }
        }
    }

    /**
     * Returns a list of delimited messages from input stream.
     *
     * @param input
     * @param delimiter
     * @return
     * @throws java.io.IOException
     */
    private List<String> getMessagesFromFile(InputStream input, String delimiter) throws IOException {
        List<String> msgs = new ArrayList<String>();
        boolean moreMessages = true;
        String s = streamToString(input);
        String sMsg = s;
        while (moreMessages)
        {
            int sepPos = s.indexOf(delimiter);
            if (sepPos > 0)
            {
                sMsg = s.substring(0, sepPos);
                s = s.substring(sepPos + 1);
            }
            else
            {
                moreMessages = false;
            }
            msgs.add(sMsg);
        }
        numOfMessages = msgs.size();
        return msgs;
    }

    /**
     * Populates INPUT.Q with supplied messages using a direct connection
     * to OpenMQ broker.
     *
     * @param messages
     * @throws javax.naming.NamingException
     * @throws javax.jms.JMSException
     */
    private void populateUsingMQConnection(List<String> messages) {
        try {
            logger.log(Level.INFO,
                "populating directly using sun queue connection factory");
            initForOpenMQ();
            TextMessage msg;
            for(String mess : messages) {
                msg = sess.createTextMessage(mess);
                messageSender.send(msg);
            }
        } catch (JMSException ex) {
            logger.log(Level.SEVERE, null, ex);
        } finally {
            closeMQResources();
        }
    }

    /**
     * Populates INPUT.Q with supplied messages via GlassFish
     * QueueConnectionFactory.
     *
     * @param messages
     * @throws javax.naming.NamingException
     * @throws javax.jms.JMSException
     */
    private void populateWithGFConnectionFactory(List<String> messages)
            throws NamingException, JMSException {
        logger.log(Level.INFO,
                "populating using GF connection factory");
        initForGF();
        for(String msg : messages) {
            message = session.createTextMessage(msg);
            messageProducer.send(message);
        }
    }

    /**
	 * Converts an InputStream into a string
	 * @param in
	 * @return the string
	 * @throws IOException
	 */
	private String streamToString (InputStream in) throws IOException {
		StringBuffer out = new StringBuffer();
		byte[] b = new byte[4096];
		for (int n; (n = in.read(b)) != -1;) {
			out.append(new String(b, 0, n));
		}
		return out.toString();
	}

    /**
     * Populates the INPUT.Q with simple text messages
     */
    private void populateWithTestMessages() {
        try {
            initForGF();
            for (int i = 1; i <= numOfMessages; i++) {
                message = session.createTextMessage("tester message " + i);
                messageProducer.send(message);
            }
            logger.log(Level.INFO,
                    "completed writing "+numOfMessages+" to the input.q");
        } catch (JMSException ex) {
            logger.log(Level.SEVERE, null, ex);
        } catch (NamingException ex) {
            logger.log(Level.SEVERE, null, ex);
        } finally {
            closeGFResources();
        }
    }

    /**
     * Initialises session and connection used to populate messages to INPUT.Q
     *
     * @throws javax.naming.NamingException
     * @throws javax.jms.JMSException
     */
    private void initForGF() throws NamingException, JMSException {
        incx = new InitialContext();
        connectionFactory =
                (QueueConnectionFactory)incx.lookup("jms/XA_Connection_Factory");
        connection = connectionFactory.createQueueConnection();
        session = connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
        q = (Queue) incx.lookup("jndi/INPUT.Q");
        messageProducer = session.createProducer((Destination)q);
        connection.start();
    }

    /**
     * Initialises session, connection and queue sender used to populate INPUT.Q
     *
     * @throws javax.jms.JMSException
     */
    private void initForOpenMQ() throws JMSException {
        com.sun.messaging.QueueConnectionFactory connFactory =
                new com.sun.messaging.QueueConnectionFactory();

		connFactory.setProperty(ConnectionConfiguration.imqAddressList,
			                      openMQHost+":"+openMQPort);
        Queue targetQueue = new com.sun.messaging.Queue("INPUT.Q");
        conn = connFactory.createQueueConnection("admin", "admin");
        sess = conn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
        conn.start();
        messageSender = sess.createSender(targetQueue);
    }

    /**
     * Closes open messageProducer, queueSession and queueConnection.
     */
    private void closeGFResources() {
        try {
            if(messageProducer!=null)
                messageProducer.close();
            if(session!=null)
                session.close();
            if(connection!=null)
                connection.close();
        } catch (JMSException ex) {
            logger.log(Level.SEVERE, null, ex);
        }
    }

    /**
     * Closes open messageSender, queueSession and queueConnection.
     */
    private void closeMQResources() {
        try {
            if(messageSender!=null)
                messageSender.close();
            if(sess!=null)
                sess.close();
            if(conn!=null)
                conn.close();
        } catch (JMSException ignore) {
            logger.log(Level.SEVERE, null, ignore);
        }
    }

}
