/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package standalonejmsconsumer;

import com.sun.messaging.ConnectionConfiguration;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.jms.JMSException;
import javax.jms.Queue;

/**
 * Standalone J2SE test for concurrent consumption of messages from an OpenMQ
 * broker. This is a crude test case, where consumer will continue to consume
 * from the input queue in a infinite loop, ignoring exceptions caused by port
 * conflicts or refused connections.
 *
 * @author slossr1
 */
public class ConcurrentJMSConsumptionJ2SETest {

    private int numOfThreads = 10;
    private com.sun.messaging.QueueConnectionFactory connectionFactory = null;
    private Queue inputQueue = null;
    private String user = "guest";
    private String password = "guest";
    private String openMQHost = "localhost";
    private String openMQPort = "7777";

    /**
     * @param args the command line arguments
     */
    public static void main(String[] args) {
        ConcurrentJMSConsumptionJ2SETest me = new ConcurrentJMSConsumptionJ2SETest();
        if(args!=null || args.length>0) {
            me.processArgsForThreads(args);
        }
        try {
            me.init();
            me.runTest();
        } catch (JMSException ex) {
            Logger.getLogger(ConcurrentJMSConsumptionJ2SETest.class.getName()).log(Level.SEVERE,
                    "JMSException encountered, exiting in error", ex);
            System.exit(1);
        }
    }

    private void init() throws JMSException {
        connectionFactory = new com.sun.messaging.QueueConnectionFactory();

        connectionFactory.setProperty(ConnectionConfiguration.imqAddressList,
                              openMQHost+":"+openMQPort);

        System.out.println("initialising input queue");
        inputQueue = new com.sun.messaging.Queue("INPUT.Q");
    }

    private void runTest() {
        for(int i=1; i<=numOfThreads; i++) {
            OpenMQConsumer mqConsumer = new OpenMQConsumer(connectionFactory,inputQueue,user,password);
            mqConsumer.run();
        }
    }

    private void processArgsForThreads(String[] args) {
        if(args==null || args.length==0) {
            return;//continue with defaults
        }
        if(args.length==1 && args[0].equals("?") || args[0].equals("help")) {
            printUsageAndExit(false);
        }
        if(args.length!=2 && args[0].equals("--t")) {
            printUsageAndExit(true);
        }
        try {
            numOfThreads = Integer.parseInt(args[1]);
        } catch (NumberFormatException nfe) {
            System.err.println(nfe);
            printUsageAndExit(true);
        }
    }

    private void printUsageAndExit(boolean incorrectUse) {
        StringBuilder sb = new StringBuilder();
        if(incorrectUse) {
            sb.append("Incorrect usage detected, u");
        } else {
            sb.append("U");
        }
        sb.append("sage is limited to a single optional argument: [--t numberOfThreads]");
        System.out.println(sb.toString());
        System.exit(0);
    }

    public int getNumOfThreads() {
        return numOfThreads;
    }

    public void setNumOfThreads(int numOfThreads) {
        this.numOfThreads = numOfThreads;
    }
}
