/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package standalonejmsconsumer;

import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.Queue;
import javax.jms.QueueConnection;
import javax.jms.QueueSession;
import javax.jms.Session;
import javax.jms.TextMessage;

import com.sun.messaging.QueueConnectionFactory;

/**
 *
 * @author slossr1
 */
public class OpenMQConsumer implements Runnable {

	private QueueConnectionFactory connectionFactory = null;
	private Queue inputQueue = null;
	private String user = null, password = null;
    private	MessageConsumer consumer = null;
	private QueueSession session = null;
	private QueueConnection connection = null;


	public OpenMQConsumer(QueueConnectionFactory connectionFactory, Queue inputQueue, String user, String password) {
		this.connectionFactory = connectionFactory;
		this.inputQueue = inputQueue;
		this.user = user;
		this.password = password;
	}

	public void run() {
		System.out.println("prompt continuously in infinite loop");
		while (true) {
			readAndPrintMessage();
		}
	}

    private void closeResources() {
        if(consumer!=null) {
             try {
                 consumer.close();
            } catch (JMSException e) {
                System.err.println(e);
            }
         }
         if(session!=null) {
             try {
                 session.close();
             } catch (JMSException e) {
                 System.err.println(e);
             }
         }
         if(connection !=null)
         {
             try
             {
                 connection.close();
             }
             catch(JMSException e)
             {
                 System.err.println(e);
             }
         }
    }

	private boolean readAndPrintMessage() {
		String msg = readSingleMsgFromQueue();
		if(msg!=null) {
			System.out.println("read messsage: "+msg);
		}
		return msg==null;
	}

    /**
     * Reads a message from queue in a tolerant manner, continues to execute if
     * encounters exceptions, to avoid application failing due to port conflicts etc
     * @return Message consumed
     */
	public String readSingleMsgFromQueue() {
	    try
	    {
	    	connection = getConnection();
	    	session = createQueueSession(connection);
	    	connection.start();

	    	consumer = session.createConsumer(inputQueue);
    		  Message m = consumer.receive(5);
    		  if (m != null) {
    		    return getMessageContent(m);
    		  } else { //try again
    		    	Message mes = consumer.receive(1500);//longer wait
    		    	if(mes != null) {
    		    		return getMessageContent(mes);
    		    	} else {
    		    		System.out.println("No more messages were found.");
    		    		return null;
    		    	}
    		    }
	    } catch(JMSException e) {
	    	System.err.println("error whilst consuming a single message from queue");
	    	e.printStackTrace();
	    	//do not exit, may be a port conflict, continue prompting..
	    }
	    finally
	    {
	         closeResources();
	    }
    	return null;
    }

    private String getMessageContent(Message m) throws JMSException {
        if (m instanceof TextMessage) {
          TextMessage message = (TextMessage) m;
          return(message.getText());
        } else {
            throw new UnsupportedOperationException("encountered non text message,  "+m);
        }
    }

	private QueueConnection getConnection() throws JMSException {

		//System.out.println("about to create a queue connection with user = "+user + ", password = "+password);

		return connectionFactory.createQueueConnection(user, password);
    }

    private QueueSession createQueueSession(QueueConnection connection) throws JMSException {

    	//System.out.println("creating an un-transacted queue session");

		return connection.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
    }

}

