/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */

package statelesssessionconsumerclient;

import com.example.company.ejb.StatelessSessionConsumerEjbBeanRemote;
import java.rmi.RemoteException;
import java.util.Random;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.InitialContext;
import javax.naming.NamingException;

/**
 *
 * @author slossr1
 */
public class EJBPrompter implements Runnable{

    /**
     * Prompts the StatelessSessionConsumerEJB to read messages from the input queue.
     * Continues prompting until it is consistently not retrieving any messages.
     */
    public void run() {
        try {
            InitialContext ctx = new InitialContext();
            //Note: this lookup is specific to jndi naming convention used by GlassFish
            StatelessSessionConsumerEjbBeanRemote remoteI = (StatelessSessionConsumerEjbBeanRemote)
                    ctx.lookup(StatelessSessionConsumerEjbBeanRemote.class.getName());
            promptUntilExhaustedQueue(remoteI);
        } catch (NamingException ex) {
            Logger.getLogger(EJBPrompter.class.getName()).log(Level.SEVERE, 
                    "NamingException encountered, ensure application is deployed and jndi names " +
                    "are configured correctly, exiting in error", ex);
            System.exit(1);
        } catch (RemoteException ex) {
            Logger.getLogger(EJBPrompter.class.getName()).log(Level.SEVERE, 
                    "JMSException encountered, exiting in error", ex);
            System.exit(1);
        }
    }

    /**
     * Prompts remote consumer ejb until queue is exhausted of messages, tollerates three
     * null reads after random interval wait, before ceasing to prompt further.
     * @param bean StatelessSessionConsumerEjbBeanRemote
     * @throws java.rmi.RemoteException
     * @throws java.lang.InterruptedException
     */
    private void promptUntilExhaustedQueue(StatelessSessionConsumerEjbBeanRemote bean) throws RemoteException {
		boolean moreMessages = true;
		while (moreMessages) {
			moreMessages = prompt(bean);
			if(!moreMessages) {
				long randomWait = 500 + (new Random()).nextInt(500);
				moreMessages = promtAfterInteval(bean, randomWait);
				if(!moreMessages) {
					randomWait = 1000 + (new Random()).nextInt(1000);
					moreMessages = promtAfterInteval(bean, randomWait);
					if(!moreMessages) {
						randomWait = 2000 + (new Random()).nextInt(2000);
						moreMessages = promtAfterInteval(bean, randomWait);
					}
				}

			}
		}
	}

	private boolean promtAfterInteval(StatelessSessionConsumerEjbBeanRemote bean, long waitTime) throws RemoteException{
		synchronized (this) {
			try {
                this.wait(waitTime);
            } catch(InterruptedException e) {
                //ignore
            }
		}
		return bean.consumeFromQueue();
	}

	private boolean prompt(StatelessSessionConsumerEjbBeanRemote bean) throws RemoteException {
		return bean.consumeFromQueue();
	}
}
